/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data IO Client module: IOClient Posix API
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ioclient-posix.h,v 1.6 2005/05/10 09:19:23 badino Exp $
 *  Release: $Name: glite-data-io-client_R_1_5_2 $
 *
 */

#ifndef GLITE_DATA_IO_CLIENT_IOCLIENT_POSIX_H_
#define GLITE_DATA_IO_CLIENT_IOCLIENT_POSIX_H_

#include "glite/data/io/client/iotypes.h"
#include <unistd.h>
#include <fcntl.h>

#ifdef __cplusplus
extern "C" {
#endif //__cplusplus

/**
 * Open a Remote File
 * @param pathname [IN] the name of the remote file to open. It could be in the 
 * format lfn://&lt;file_name&gt; or guid:&lt;file_guid&gt;. glite_open also 
 * accept the file name without the prefix: In that case if file_name starts
 * with a '/' is interpreted as an LFN; otherwise if it's a valid GUID string 
 * representation, it's interpreted as a GUID
 * @param flags [IN] the requested access flag. The value should be a the 
 * bitwise inclusive-OR of values from the following lists: <BR>
 * O_RDONLY - open for reading only <BR>
 * O_WRONLY - open for writing only <BR>
 * O_CREAT  - create a file <BR>
 * @param mode [IN] specifies what permissions the file has when it is created
 * @return On success, the new file descriptor. On error, -1 is returned, and 
 * glite_posix_errno is set appropriately
 */
int glite_posix_open(const char * pathname, int flags, mode_t mode);

/**
 * Create a Remote File. 
 * This call is equivalent to glite_posix_open(pathname, O_CREAT, mode)
 * @param pathname [IN] the name of the remote file to open. It could be in the 
 * format lfn://&lt;file_name&gt; or guid:&lt;file_guid&gt;. glite_creat also 
 * accept the file name without the prefix: In that case if file_name starts
 * with a '/' is interpreted as an LFN; otherwise if it's a valid GUID string 
 * representation, it's interpreted as a GUID
 * @param mode [IN] specifies what permissions the file has when it is created
 * @return On success, the new file descriptor. On error, -1 is returned, and 
 * glite_posix_errno is set appropriately
 */
int glite_posix_creat(const char * pathname, mode_t mode);

/**
 * Read up to count bytes from file descriptor fd into the buffer starting 
 * at buf
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open 
 * call
 * @param buf [IN] the buffer where to store the data
 * @param count [IN] the number of byte to read. This parameter should be equal 
 * or less the size of the buffer
 * @return On success, the number of bytes read is returned (zero indicates 
 * end of file), and the file position is advanced by this number. On error, -1 
 * is returned, and glite_posix_errno is set appropriately
 */
ssize_t glite_posix_read(int fd, void * buf, size_t count);

/**
 * Writes up to count bytes to the file referenced by the file descriptor fd 
 * from the buffer starting at buf.
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open or 
 * glite_posix_creat call
 * @param buf [IN] the buffer containing the data to be written
 * @param count [IN] the number of byte to write. This parameter should be equal 
 * or less the size of the buffer
 * @return On success, the number of bytes written are returned (zero indicates
 * nothing was written). On error, -1 is returned, and glite_posix_errno is set
 * appropriately. 
 */
ssize_t glite_posix_write(int fd, const void * buf, size_t count);

/**
 * Set a file read/write pointer
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open or 
 * glite_posix_creat call
 * @param offset [IN] new byte offset to seek to
 * @param whence [IN] relative file position. it can be one of the following 
 * values: <BR>
 * SEEK_SET - set to offset <BR>
 * SEEK_CUR - set to current position plus offset <BR>
 * SEEK_END - set to the size of the file plus offset <BR>
 * @return the resulting offset location as  measured  in bytes from the 
 * beginning of the file. Otherwise, a value of (off_t)-1 is returned and 
 * glite_posix_errno is set to indicate the error
 */
off_t glite_posix_lseek(int fd, off_t offset, int whence);

/**
 * Set a file read/write pointer
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open or 
 * glite_posix_creat call
 * @param offset [IN] new byte offset to seek to
 * @param whence [IN] relative file position. it can be one of the following 
 * values: <BR>
 * SEEK_SET - set to offset <BR>
 * SEEK_CUR - set to current position plus offset <BR>
 * SEEK_END - set to the size of the file plus offset <BR>
 * @return the resulting offset location as  measured  in bytes from the 
 * beginning of the file. Otherwise, a value of (off64_t)-1 is returned and 
 * glite_posix_errno is set to indicate the error
 */
off64_t glite_posix_lseek64(int fd, off64_t offset, int whence);

/**
 * Return information about the specified file
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open or 
 * glite_posix_creat call
 * @param buf [OUT] pointer to a stat structure, as defined in <sys/stat.h>, 
 * into which information is placed concerning the file
 * @return On success, zero is returned. On error, -1 is returned, and 
 * glite_posix_errno is set appropriately
 */
int glite_posix_fstat(int fd, struct stat *buf);

/**
 * Return information about the specified file. To be used for large files
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open or 
 * glite_posix_creat call
 * @param buf [OUT] pointer to a stat structure, as defined in <sys/stat.h>, 
 * into which information is placed concerning the file
 * @return On success, zero is returned. On error, -1 is returned, and 
 * glite_posix_errno is set appropriately
 */
int glite_posix_fstat64(int fd, struct stat64 *buf);
  
/**
 * Close the file
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open or 
 * glite_posix_creat call
 * @return On success, zero is returned. On error, -1 is returned, and 
 * glite_posix_errno is set appropriately
 */
int glite_posix_close(int fd);


/**
 * Deletes a file on the remote Storage element. That replica information will 
 * be removed from the Logical File System and an attempt is made to remove the 
 * physical instance, but if that fails no error are returned. 
 * In case that replica is the last one, also the file logical name and GUID 
 * will be removed, so they can be reused later
 * <b>TODO</b> <i>Define GLite Error Codes (now the aiod ones are used)</i><BR>
 * @param pathname [IN] the name of the remote file to delete. It could be in 
 * the format lfn://&lt;file_name&gt; or guid:&lt;file_guid&gt;. glite_creat also 
 * accept the file name without the prefix: In that case if file_name starts
 * with a '/' is interpreted as an LFN; otherwise if it's a valid GUID string 
 * representation, it's interpreted as a GUID
 * @return On success, zero is returned. On error, -1 is returned, and 
 * glite_posix_errno is set appropriately
 */
int glite_posix_unlink(const char * pathname);

/**
 * Returns the file handle associated with the descriptor. The returned handle 
 * could be used to call glite-io non-posix methods (like glite_fstat), but 
 * calling glite_close on a file opened with glite_posix_open may cause an 
 * impredictable behavior, so should be avoided in any case
 * @param fd [IN] the file descriptor returned by a previos glite_posix_open or 
 * glite_posix_creat call
 * @return On success, the associated file handle if success. On error, 
 * GLITE_NULL_HANDLE is returned
 */
glite_handle glite_filehandle(int fd);

/**
 * Number of last error. <i>Be aware that is not thread safe !</i>
 */
extern int glite_posix_errno;
 
#ifdef __cplusplus
}
#endif //__cplusplus

#endif //GLITE_DATA_IO_CLIENT_IOCLIENT_POSIX_H_
