/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data IO Client module: IOClient API
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ioclient.h,v 1.11 2005/10/06 08:44:56 zfarkas Exp $
 *  Release: $Name: glite-data-io-client_R_1_5_2 $
 *
 */

#ifndef GLITE_DATA_IO_CLIENT_IOCLIENT_H_
#define GLITE_DATA_IO_CLIENT_IOCLIENT_H_

#include "glite/data/io/client/iotypes.h"
#include <fcntl.h>


#ifdef __cplusplus
extern "C" {
#endif //__cplusplus

/* Service discovery type of gLite I/O */
#define GLITE_IO_SD_TYPE "org.glite.gliteIO"

/* Environment variable to override Service type */
#define GLITE_IO_SD_ENV  "GLITE_SD_GLITEIO_TYPE"

/* gLite IO URL prefix */
#define GLITE_IO_URL_PREFIX "gliteio://"

/* Verbose environment variable */
#define TOOL_USER_VERBOSE "__GLITE_IO_VERBOSE"

/**
 * Check if string is in form "gliteio://..."
 * @param string [IN] the string to check
 * @return true is string is in form above, false if not
 */
bool is_gliteio_url(const char *string);

/**
 * Initialize the service
 * @param endpoint [IN] user-given service endpoint or NULL
 * @param encryptdata [IN] user-given data encryption or false
 * @return 0 on success, -1 on error
 */
int glite_io_initialize(const char *endpoint, const bool encryptdata);
      
/**
 * Finalize a service
 */
void glite_io_finalize();

/**
 * Open a Remote File
 * @param pathname [IN] the name of the remote file to open. It could be in the 
 * format lfn://&lt;file_name&gt; or guid:%lt;file_guid&gt;. glite_open also 
 * accept the file name without the prefix: In that case if file_name starts
 * with a '/' is interpreted as an LFN; otherwise if it's a valid GUID string 
 * representation, it's interpreted as a GUID
 * @param flags [IN] the requested access flag. The value should be a the 
 * bitwise inclusive-OR of values from the following lists: <BR>
 * O_RDONLY - open for reading only <BR>
 * O_WRONLY - open for writing only <BR>
 * O_CREAT  - create a file <BR>
 * @param mode [IN] specifies what permissions the file has when it is created
 * @param size [IN] specifies how many bytes should be preallocated remotely. 
 * It will be used only if the O_CREAT flag is set and may be null
 * @param result [OUT] the result code associated with the operation. This 
 * parameter is used to return additional information in case of errors and can 
 * be null if that information is not requested
 * @return a handle to the opened file, or GLITE_NULL_HANDLE in case of failure
 */
glite_handle glite_open(const char * pathname, glite_int32 flags, glite_int32 mode, glite_int64 size, glite_result* result);

/**
 * Create a Remote File. 
 * This call is equivalent to glite_open(name, O_CREAT, mode, server, options)
 * @param pathname [IN] the name of the remote file to open. It could be in the 
 * format lfn://&lt;file_name&gt; or guid:&lt;file_guid&gt;. glite_creat also 
 * accept the file name without the prefix: In that case if file_name starts
 * with a '/' is interpreted as an LFN; otherwise if it's a valid GUID string 
 * representation, it's interpreted as a GUID
 * @param mode [IN] specifies what permissions the file has when it is created
 * @param size [IN] specifies how many bytes should be preallocated remotely. 
 * May be null
 * @param result [OUT] the result code associated with the operation. This 
 * parameter is used to return additional information in case of errors and can 
 * be null if that information is not requested
 * @return a handle to the opened file, or GLITE_NULL_HANDLE in case of failure
 */
glite_handle glite_creat(const char * pathname, glite_int32 mode, glite_int64 size, glite_result* result);

/**
 * Read the file
 * @param fh [IN] the file handle returned by a previos glite_open call
 * @param buf [IN] the buffer where to store the data
 * @param count [IN] the number of byte to read. This parameter should be equal 
 * or less the size of the buffer
 * @return the number of bytes read, or a negative value in case of error
 */
glite_int32 glite_read(glite_handle fh, void * buf, glite_int32 count);

/**
 * Write to the file
 * @param fh [IN] the file handle returned by a previos glite_open or 
 * glite_creat call
 * @param buf [IN] the buffer containing the data to be written
 * @param count [IN] the number of byte to write. This parameter should be equal 
 * or less the size of the buffer
 * @return the number of bytes written, or a negative value in case of error
 */
glite_int32 glite_write(glite_handle fh, const void * buf, glite_int32 count);

/**
 * Set a file read/write pointer
 * @param fh [IN] the file handle returned by a previos glite_open or 
 * glite_creat call
 * @param offset [IN] new byte offset to seek to
 * @param whence [IN] relative file position. it can be one of the following 
 * values: <BR>
 * SEEK_SET - set to offset <BR>
 * SEEK_CUR - set to current position plus offset <BR>
 * SEEK_END - set to the size of the file plus offset <BR>
 * @return The new offset from the beginning of the file, or a negative value 
 * in case of error
 */
glite_int64 glite_lseek(glite_handle fh, glite_int64 offset, glite_int32 whence);

/**
 * Get Information about the file associated with the given handle 
 * @param fh [IN] the file handle returned by a previos glite_open or 
 * glite_creat call
 * @param buf [OUT] pointer to a stat structure, as defined in iotypes.h, 
 * into which information is placed concerning the file
 * @return GLITE_IO_SUCCESS if success, or a negative value in case of error
 */
glite_int32 glite_fstat(glite_handle fh, struct glite_stat * buf);

/**
 * Close the file
 * @param fh [IN] the file handle returned by a previos glite_open or 
 * glite_creat call
 * @return GLITE_IO_SUCCESS if success, a negative value in case of error
 */
glite_int32 glite_close(glite_handle fh);

/**
 * Deletes a file on the remote Storage element. That replica information will 
 * be removed from the Logical File System and an attempt is made to remove the 
 * physical instance, but if that fails no error are returned. 
 * In case that replica is the last one, also the file logical name and GUID 
 * will be removed, so they can be reused later
 * <b>TODO</b> <i>Define GLite Error Codes (now the aiod ones are used)</i><BR>
 * @param pathname [IN] the name of the remote file to delete. It could be in 
 * the format lfn://&lt;file_name&gt; or guid:&lt;file_guid&gt;. glite_unlink 
 * also accept the file name without the prefix: In that case if file_name 
 * starts with a '/' is interpreted as an LFN; otherwise if it's a valid GUID 
 * string representation, it's interpreted as a GUID
 * @return GLITE_IO_SUCCESS if success, a negative value in case of error
 */
glite_int32 glite_unlink(const char * pathname);

/**
 * Get the Last GLite Error code associated to a File Handle. <BR>
 * <i>This method cannot be used after an open or close call, since there's 
 * no valid file handle</i><BR>
 * <b>TODO</b> <i>Define GLite Error Codes (now the aiod ones are used)</i><BR>
 * @param fh [IN] the file handle returned by a previos glite_open or 
 * glite_creat call
 * @return GLITE_IO_SUCCESS if success, a negative value in case of error
 */
glite_int32 glite_error(glite_handle fh);

/**
 * Get a string representing a GLite Error code
 * <b>TODO</b> <i>Define GLite Error Codes (now the aiod ones are used)</i><BR>
 * @param error [IN] the error code to be translated
 * @return the error strig
 */
const char * glite_strerror(glite_result error);

#ifdef __cplusplus
}
#endif //__cplusplus

#endif //GLITE_DATA_IO_CLIENT_IOCLIENT_H_
